<?php
/**
 * Created by PhpStorm.
 * User: MATHEUSSILVA
 * Date: 26/03/2015
 * Time: 11:17
 */

namespace Folha\Logic\ObrigacaoLegal\RAIS;

use \Folha\Entities\GpePessoah;

use \Folha\Entities\GpeVinculoh;

use \Folha\Logic\ObrigacaoLegal\RAIS\Entities\Tipo00Base;

use \Geral\Entities\Comunicaparc;

use \Geral\Entities\Enderecoparc;

use \Geral\Entities\Estruturah;

use \Geral\Entities\Estruturam;

use \HCM\Util\DateUtil;

class Tipo00 extends Tipo00Base
{
    private $nrorg;
    private $ano;
    /** @var  $tipo01 Tipo01[] */
    private $tipo01;
    /** @var  $tipo09 Tipo09 */
    private $tipo09;
    /** @var  int $qtd02 */
    private $qtd02;
    /** @var  int $nrformrais */
    private $nrformrais;



    /**
     * Construtor
     * @param Estruturah $responsEstrut
     * @param GpeVinculoh $responsVinc
     * @param bool $retifica
     */
    function __construct( $responsEstrut, $responsVinc, $nrorg, $ano, $nrformrais, $retifica = false)
    {
        $this->nrorg = $nrorg;
        $this->ano = $ano;
        $this->setDefaultInfo($retifica);
        $this->setRespDataVinc($responsVinc);
        $this->setRespDataEst($responsEstrut);
        $this->qtd02 = 0;
        $this->nrformrais = $nrformrais;
    }

    /**
     * Seta as informações da primeira empresa na RAIS tipo 00
     */
    private function setPriEmpresaInfo()
    {
        $info = $this->tipo01[0]->getInfoZero();
        $this->cnpjPrimeiro = $info['CNPJ'];
        $this->prefixo = $info['PREFIXO'];

    }

    /**
     * Seta as informações padrões
     * @param bool $retifica
     */
    private function setDefaultInfo($retifica)
    {
        $this->espacos = '';
        $this->espacos2 = '';
        $this->tipoRegistro = 0;
        $this->constante = 1;
        $this->dataGeracao = DateUtil::getDataAtual();
        if ($retifica == 'S') {
            $this->retificacao = 1;
            $this->dataRetificacao = DateUtil::getDataAtual();
        } else {
            $this->retificacao = 2;
        }
    }

    /**
     * Seta os dados do responsável, se o responsável for uma pessoa
     * @param $pessoah GpePessoah
     * @throws \Exception
     */
    private function setRespDataVinc( $pessoah)
    {
        try {
            //BUSCA A FORMA DE CONTATO COM O RESPONSAVEL
            $where = "NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND NRORG = :P_NRORG AND CDFORMACOMU = :P_CDFORMACOMU";
            $params = array(
                'P_NRPARCNEGOCIO' => $pessoah->getPessoa()->getNrparcnegocio(),
                'P_NRORG' => $this->nrorg,
                'P_CDFORMACOMU' => self::FORMACOMU_EMAIL
            );
            $orderBy = "";
            $email = Comunicaparc::getObjectByWhere($where, $params, $orderBy);
            /** @var Comunicaparc $email */

            if (!$email) {
                throw Exception::newException(Exception::ERRO_EMAIL_RESPONS_RAIS);
            }

            $this->cpfResp = $pessoah->getNrcpfpessoa();
            $this->nomeResp = $pessoah->getNmpessoa();
            $this->dataNascResp = $pessoah->getDtnascpessoa();
            $this->emailResp = $email->getCdcomunicaparc();
            if ($pessoah->getNrconselhoreg() == self::NUM_ID_CREA) {
                $this->creaRetif = $pessoah->getNrinscrconsreg();
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * Seta os dados do responsável, se o responsável for uma estrutura
     * @param Estruturah $estruturah
     * @throws \Exception
     */
    private function setRespDataEst( $estruturah)
    {
        try {
            //BUSCA OS DADOS DA ESTRUTURA QUE SERÁ O RESPONSÁVEL PELA RAIS
            $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG";
            $params = array(
                'P_NRESTRUTURAM' => $estruturah->getNrestruturam(),
                'P_NRORG' => $estruturah->getNrorg(),
            );
            $estruturam = Estruturam::getObjectByWhere($where, $params);
            /** @var Estruturam $estruturam */

            //BUSCA OS DADOS DE ENDEREÇO DA ESTRUTURA QUE SERÁ RESPONSÁVEL PELA RAIS
            $where = "NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND NRORG = :P_NRORG";
            $params = array(
                'P_NRPARCNEGOCIO' => $estruturam->getNrparcnegocio(),
                'P_NRORG' => $this->nrorg
            );
            $endereco = Enderecoparc::getObjectByWhere($where, $params);
            /** @var Enderecoparc $endereco */

            //BUSCA A FORMA DE CONTATO COM O RESPONSAVEL
            $where = "NRPARCNEGOCIO = :P_NRPARCNEGOCIO AND NRORG = :P_NRORG AND CDFORMACOMU = :P_CDFORMACOMU";
            $params = array(
                'P_NRPARCNEGOCIO' => $estruturam->getNrparcnegocio(),
                'P_NRORG' => $this->nrorg,
                'P_CDFORMACOMU' => self::FORMACOMU_TEL
            );
            $telefone = Comunicaparc::getObjectByWhere($where, $params);
            /** @var Comunicaparc $telefone */

            if (!is_null($estruturah->getCdcnpjestrut())) {
                $this->cnpj = $estruturah->getCdcnpjestrut();
                $this->tipoInscricao = 1;
            } else {
                $this->cnpj = $estruturah->getCdceiestrut();
                $this->tipoInscricao = 3;
            }
            $this->razaoSocial = $estruturah->getNmrazsocestrut();
            if ($endereco) {
                $this->logradouro = $endereco->getDsendereco();
                $this->numero = $endereco->getNrenderecoparc();
                $this->complemento = $endereco->getDscompleendereco();
                $this->uf = $endereco->getSgestado();
                $this->cep = $endereco->getNrcependereco();
                $this->bairro = $endereco->getNmbairroendereco();
                if ($endereco->getCdmunicipio()) {
                    $this->codMunicipio = $endereco->getMunicipio()->getCdmunicibge();
                    $this->municipio = $endereco->getMunicipio()->getNmmunicipio();
                }else{
                    $params = array(
                        $estruturam->getNrestruturam(),
                        $estruturam->getNmestruturam()
                    );
                    Exception::newException(Exception::ALERTA_MUNICIPIO_ESTRUTURA, $params, Exception::EXCEPTION_ALERTA);
                }
            } else {
                throw Exception::newException(Exception::ERRO_ENDER_RESPONS_RAIS);
            }
            if ($telefone) {
                $this->ddd = $telefone->getCdprefixcomuparc();
                $this->numeroTel = $telefone->getCdcomunicaparc();
            }else{
                throw Exception::newException(Exception::ERRO_TELEF_RESPONS_RAIS);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Seta as estruturas que serão contempladas na RAIS
     * @param int $centralPgto
     * @param int $sindicato
     * @param String $nrestruturasm
     * @throws \Exception
     */
    public function setTipo01( $nrestruturasm = null, $centralPgto = null, $sindicato = null)
    {
        try {
            $where = "NRORG = :P_NRORG AND NRTIPOESTRUTURA = :P_NRTIPOESTRUTURA" . ($nrestruturasm ? " AND NRESTRUTURAM IN ($nrestruturasm)" : "");
            $params = array(
                'P_NRORG' => $this->nrorg,
                'P_NRTIPOESTRUTURA' => \FPA::getTipoEstruturaLegalOrg()
            );
            $estruturas = Estruturam::getObjectsByWhere($where, $params);
            /** @var Estruturam[] $estruturas */
            if ($estruturas) {
                foreach ($estruturas as $estruturam) {
                    $where = "NRESTRUTURAM = :P_NRESTRUTURAM AND NRORG = :P_NRORG";
                    $params = array(
                        'P_NRESTRUTURAM' => $estruturam->getNrestruturam(),
                        'P_NRORG' => $estruturam->getNrorg()
                    );
                    $orderBy = "DTMESCOMPETENC DESC";
                    $estruturah = Estruturah::getObjectByWhere($where, $params, $orderBy);
                    /** @var Estruturah $estruturah */

                    if(!$estruturah){
                        $params = array(
                            $estruturam->getNrestruturam(),
                            $estruturam->getNmestruturam()
                        );
                        throw Exception::newException(Exception::ERRO_ESTRUTURAH_NAO_LOCA, $params);
                    }
                    if ($estruturam->getNrestruturam() == $centralPgto) {
                        $centralPgto = true;
                    }

                    $this->tipo01[] = new Tipo01($estruturam, $estruturah, $this->ano, $centralPgto, $sindicato, $this->nrformrais);
                }
                $this->setPriEmpresaInfo();
            }else{
                throw Exception::newException(Exception::ERRO_ESTRUTURAM_NAO_LOCA, $params);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function setTipo02($grupoIncidencias, $nrvinculos)
    {
        foreach($this->tipo01 as $tipo01){
            $tipo01->setRais02($grupoIncidencias, $nrvinculos);
            $this->qtd02 += $tipo01->getQuantDois();
        }
    }

    public function setTipo09()
    {
        $tipo01 = end($this->tipo01);
        $info = $tipo01->getInfoZero();
        $this->tipo09 = new Tipo09($info['CNPJ'], count($this->tipo01), $this->qtd02, $info['PREFIXO']);

    }


    public function getMagnetico()
    {
        $linhas = array();
        $linhas[] = $this->getRow();
        foreach($this->tipo01 as $tipo01){
            $linhas[] = $tipo01->getRow();
            $tipo02mags = $tipo01->getTipo02();
            foreach($tipo02mags as $tipo02mag){
                $linhas[] = $tipo02mag;
            }
        }
        $linhas[] = $this->tipo09->getRow();
        return $linhas;
    }

} 